@@ -15,14 +15,30 @@ for name, obj in inspect.getmembers(instructions, inspect.isclass):
 
 def assemble(lines):
 
-    stripped_lines = filter(lambda x: x, strip_comments(map(str.strip, lines)))
+    stripped_lines = strip_comments(map(str.strip, lines))
+    tokenized_lines = [(line_number, scanner.scan(line)) for
+                       line_number, line in stripped_lines]
 
-    tokenized_lines = map(scanner.scan, stripped_lines)
+    # nop insertion pass
+    hot_register = None
+    for i, line_tuple in enumerate(tokenized_lines):
+        line_number, line = line_tuple
+        tokens, rest = line
+        token_type, value = tokens[0]
+        if token_type == 'operator':
+            if value == 'ld':
+                hot_register = tokens[1][1]
+            else:
+                for token in tokens[1:]:
+                    token_type, value = token
+                    if token_type == 'register' and value == hot_register:
+                        tokenized_lines.insert(i, ([('operator', 'nop')], ''))
+                        hot_register = None
 
     # simple label pass
     labels = {}
     current_address = 1
-    for line in tokenized_lines:
+    for line_number, line in tokenized_lines:
         tokens, rest = line
         token_type, value = tokens[0]
         if token_type == 'label':
@@ -33,9 +49,7 @@ def assemble(lines):
 
     #actual assemble pass
     assembly = []
-    line_number = 0
-    for line in tokenized_lines:
-        line_number += 1
+    for line_number, line in tokenized_lines:
         tokens, rest = line
         if rest:
             print '[%s]: Tokenizer error near' % line_number, rest
@@ -53,10 +67,16 @@ def assemble(lines):
                 if token_type == 'if':
                     token = tokens.next()
                     token_type, value = token
+                    prefix = ''
+                    if token == ('operator', 'not'):
+                        token = tokens.next()
+                        token_type, value = token
+                        prefix = 'not '
                     if token_type == 'condition':
-                        condition = Cond.fromString(value)
+                        condition = Cond.fromString(prefix + value)
                     else:
-                        print 'Your if no sense. Line number:', line_number
+                        print 'Your if makes no sense. Line number:',
+                        print line_number
 
                 elif token_type == 'operator':
 
@@ -80,7 +100,6 @@ def assemble(lines):
                         if maybe_label == 'label':
                             b = labels[a]
                             a = 0
-                            print instruction, a, b
                         else:
                             _, b = tokens.next()
                         instruction = instruction_map[value.capitalize()](a, b)
@@ -116,9 +135,15 @@ def assemble(lines):
 
 def strip_comments(lines):
     newline_delimitor = '!newline delimitor!'
+    line_number_delimitor = '!line number delimitor!'
+    lines = [str(line_number) + line_number_delimitor + line for
+             line_number, line in enumerate(lines)]
     joined = newline_delimitor.join(lines)
     stripped = re.sub('(/\*([^*]|(\*+[^*/]))*\*+/)', '', joined)
-    return stripped.split(newline_delimitor)
+    annotated = filter(lambda x: x[1],
+                       [line.split(line_number_delimitor)
+                        for line in stripped.split(newline_delimitor)])
+    return [(int(line_number), line) for line_number, line in annotated]
 
 
 def ascii_binary_to_real_binary(ascii_binary):
